@@ -97,10 +97,11 @@ function start() {
     Shell.WindowTracker.get_default();
     Shell.AppUsage.get_default();
 
-    // The background color really only matters if there is no desktop
-    // window (say, nautilus) running. We set it mostly so things look good
-    // when we are running inside Xephyr.
+    // The stage is always covered so Clutter doesn't need to clear it; however
+    // the color is used as the default contents for the Mutter root background
+    // actor so set it anyways.
     global.stage.color = DEFAULT_BACKGROUND_COLOR;
+    global.stage.no_clear_hint = true;
 
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
     let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
@@ -161,10 +162,6 @@ function start() {
         }
     });
 
-    background = global.create_root_pixmap_actor();
-    global.stage.add_actor(background);
-    background.lower_bottom();
-
     global.gdk_screen.connect('monitors-changed', _relayout);
 
     ExtensionSystem.init();
@@ -240,8 +237,6 @@ function _relayout() {
     panel.actor.set_size(primary.width, Panel.PANEL_HEIGHT);
     overview.relayout();
 
-    background.set_size(global.screen_width, global.screen_height);
-
     // To avoid updating the position and size of the workspaces
     // in the overview, we just hide the overview. The positions
     // will be updated when it is next shown. We do the same for
@@ -390,6 +385,7 @@ function pushModal(actor) {
     }
     modalActorFocusStack.push([actor, curFocus]);
 
+    global.stage.set_key_focus(null);
     return true;
 }
 
