@@ -1,6 +1,8 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
+const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
@@ -14,6 +16,7 @@ const Tweener = imports.ui.tweener;
 const Main = imports.ui.main;
 const BoxPointer = imports.ui.boxpointer;
 const Params = imports.misc.params;
+const Utils = imports.misc.utils;
 
 const ANIMATION_TIME = 0.2;
 const NOTIFICATION_TIMEOUT = 4;
@@ -22,8 +25,6 @@ const SUMMARY_TIMEOUT = 1;
 const HIDE_TIMEOUT = 0.2;
 const LONGER_HIDE_TIMEOUT = 0.6;
 
-const BUTTON_ICON_SIZE = 36;
-
 const MAX_SOURCE_TITLE_WIDTH = 180;
 
 // We delay hiding of the tray if the mouse is within MOUSE_LEFT_ACTOR_THRESHOLD
@@ -37,15 +38,132 @@ const State = {
     HIDING:  3
 };
 
-function _cleanMarkup(text) {
-    // Support &amp;, &quot;, &apos;, &lt; and &gt;, escape all other
-    // occurrences of '&'.
-    let _text = text.replace(/&(?!amp;|quot;|apos;|lt;|gt;)/g, '&amp;');
-    // Support <b>, <i>, and <u>, escape anything else
-    // so it displays as raw markup.
-    return _text.replace(/<(\/?[^biu]>|[^>\/][^>])/g, '&lt;$1');
+function _fixMarkup(text, allowMarkup) {
+    if (allowMarkup) {
+        // Support &amp;, &quot;, &apos;, &lt; and &gt;, escape all other
+        // occurrences of '&'.
+        let _text = text.replace(/&(?!amp;|quot;|apos;|lt;|gt;)/g, '&amp;');
+        // Support <b>, <i>, and <u>, escape anything else
+        // so it displays as raw markup.
+        return _text.replace(/<(\/?[^biu]>|[^>\/][^>])/g, '&lt;$1');
+    } else {
+        // Escape everything
+        let _text = text.replace(/&/g, '&amp;');
+        return _text.replace(/</g, '&lt;');
+    }
 }
 
+function URLHighlighter(text, lineWrap, allowMarkup) {
+    this._init(text, lineWrap, allowMarkup);
+}
+
+URLHighlighter.prototype = {
+    _init: function(text, lineWrap, allowMarkup) {
+        if (!text)
+            text = '';
+        this.actor = new St.Label({ reactive: true, style_class: 'url-highlighter' });
+        this._linkColor = '#ccccff';
+        this.actor.connect('style-changed', Lang.bind(this, function() {
+            let color = new Clutter.Color();
+            let hasColor = this.actor.get_theme_node().get_color('link-color', color);
+            if (hasColor) {
+                let linkColor = color.to_string().substr(0, 7);
+                if (linkColor != this._linkColor) {
+                    this._linkColor = linkColor;
+                    this._highlightUrls();
+                }
+            }
+        }));
+        if (lineWrap) {
+            this.actor.clutter_text.line_wrap = true;
+            this.actor.clutter_text.line_wrap_mode = Pango.WrapMode.WORD_CHAR;
+            this.actor.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
+        }
+
+        this.setMarkup(text, allowMarkup);
+        this.actor.connect('button-release-event', Lang.bind(this, function (actor, event) {
+            let urlId = this._findUrlAtPos(event);
+            if (urlId != -1) {
+                let url = this._urls[urlId].url;
+                if (url.indexOf(':') == -1)
+                    url = 'http://' + url;
+                try {
+                    Gio.app_info_launch_default_for_uri(url, global.create_app_launch_context());
+                    return true;
+                } catch (e) {
+                    // TODO: remove this after gnome 3 release
+                    let p = new Shell.Process({ 'args' : ['gvfs-open', url] });
+                    p.run();
+                    return true;
+                }
+            }
+            return false;
+        }));
+        this.actor.connect('motion-event', Lang.bind(this, function(actor, event) {
+            let urlId = this._findUrlAtPos(event);
+            if (urlId != -1 && !this._cursorChanged) {
+                global.set_cursor(Shell.Cursor.POINTING_HAND);
+                this._cursorChanged = true;
+            } else if (urlId == -1) {
+                global.unset_cursor();
+                this._cursorChanged = false;
+            }
+            return false;
+        }));
+        this.actor.connect('leave-event', Lang.bind(this, function() {
+            if (this._cursorChanged) {
+                this._cursorChanged = false;
+                global.unset_cursor();
+            }
+        }));
+    },
+
+    setMarkup: function(text, allowMarkup) {
+        text = text ? _fixMarkup(text, allowMarkup) : '';
+        this._text = text;
+
+        this.actor.clutter_text.set_markup(text);
+        /* clutter_text.text contain text without markup */
+        this._urls = Utils.findUrls(this.actor.clutter_text.text);
+        this._highlightUrls();
+    },
+
+    _highlightUrls: function() {
+        // text here contain markup
+        let urls = Utils.findUrls(this._text);
+        let markup = '';
+        let pos = 0;
+        for (let i = 0; i < urls.length; i++) {
+            let url = urls[i];
+            let str = this._text.substr(pos, url.pos - pos);
+            markup += str + '<span foreground="' + this._linkColor + '"><u>' + url.url + '</u></span>';
+            pos = url.pos + url.url.length;
+        }
+        markup += this._text.substr(pos);
+        this.actor.clutter_text.set_markup(markup);
+    },
+
+    _findUrlAtPos: function(event) {
+        let success;
+        let [x, y] = event.get_coords();
+        [success, x, y] = this.actor.transform_stage_point(x, y);
+        let find_pos = -1;
+        for (let i = 0; i < this.actor.clutter_text.text.length; i++) {
+            let [success, px, py, line_height] = this.actor.clutter_text.position_to_coords(i);
+            if (py > y || py + line_height < y || x < px)
+                continue;
+            find_pos = i;
+        }
+        if (find_pos != -1) {
+            for (let i = 0; i < this._urls.length; i++)
+            if (find_pos >= this._urls[i].pos &&
+                this._urls[i].pos + this._urls[i].url.length > find_pos)
+                return i;
+        }
+        return -1;
+    }
+};
+
 // Notification:
 // @source: the notification's Source
 // @title: the title
@@ -75,7 +193,8 @@ function _cleanMarkup(text) {
 // area.
 //
 // @params can contain values for 'customContent', 'body', 'icon',
-// and 'clear' parameters.
+// 'titleMarkup', 'bannerMarkup', 'bodyMarkup', and 'clear'
+// parameters.
 //
 // If @params contains a 'customContent' parameter with the value %true,
 // then @banner will not be shown in the body of the notification when the
@@ -89,6 +208,12 @@ function _cleanMarkup(text) {
 // source.createNotificationIcon(). However, if @params contains an 'icon'
 // parameter, the passed in icon will be used.
 //
+// If @params contains a 'titleMarkup', 'bannerMarkup', or
+// 'bodyMarkup' parameter with the value %true, then the corresponding
+// element is assumed to use pango markup. If the parameter is not
+// present for an element, then anything that looks like markup in
+// that element will appear literally in the output.
+//
 // If @params contains a 'clear' parameter with the value %true, then
 // the content and the action area of the notification will be cleared.
 // The content area is also always cleared if 'customContent' is false
@@ -105,6 +230,7 @@ Notification.prototype = {
         this._useActionIcons = false;
         this._customContent = false;
         this._bannerBodyText = null;
+        this._bannerBodyMarkup = false;
         this._titleFitsInBannerMode = true;
         this._spacing = 0;
 
@@ -150,7 +276,8 @@ Notification.prototype = {
 
         this._titleLabel = new St.Label();
         this._bannerBox.add_actor(this._titleLabel);
-        this._bannerLabel = new St.Label();
+        this._bannerUrlHighlighter = new URLHighlighter();
+        this._bannerLabel = this._bannerUrlHighlighter.actor;
         this._bannerBox.add_actor(this._bannerLabel);
 
         this.update(title, banner, params);
@@ -177,6 +304,9 @@ Notification.prototype = {
         params = Params.parse(params, { customContent: false,
                                         body: null,
                                         icon: null,
+                                        titleMarkup: false,
+                                        bannerMarkup: false,
+                                        bodyMarkup: false,
                                         clear: false });
 
         this._customContent = params.customContent;
@@ -207,7 +337,7 @@ Notification.prototype = {
                                      y_fill: false,
                                      y_align: St.Align.START });
 
-        title = title ? _cleanMarkup(title.replace(/\n/g, ' ')) : '';
+        title = title ? _fixMarkup(title.replace(/\n/g, ' '), params.titleMarkup) : '';
         this._titleLabel.clutter_text.set_markup('<b>' + title + '</b>');
 
         // Unless the notification has custom content, we save this._bannerBodyText
@@ -215,9 +345,11 @@ Notification.prototype = {
         // expandable due to other elements in its content area or due to the banner
         // not fitting fully in the single-line mode.
         this._bannerBodyText = this._customContent ? null : banner;
+        this._bannerBodyMarkup = params.bannerMarkup;
 
-        banner = banner ? _cleanMarkup(banner.replace(/\n/g, '  ')) : '';
-        this._bannerLabel.clutter_text.set_markup(banner);
+        banner = banner ? banner.replace(/\n/g, '  ') : '';
+
+        this._bannerUrlHighlighter.setMarkup(banner, params.bannerMarkup);
         this._bannerLabel.queue_relayout();
 
         // Add the bannerBody now if we know for sure we'll need it
@@ -225,7 +357,7 @@ Notification.prototype = {
             this._addBannerBody();
 
         if (params.body)
-            this.addBody(params.body);
+            this.addBody(params.body, params.bodyMarkup);
         this._updated();
     },
 
@@ -233,7 +365,7 @@ Notification.prototype = {
     // @actor: actor to add to the body of the notification
     //
     // Appends @actor to the notification's body
-    addActor: function(actor) {
+    addActor: function(actor, style) {
         if (!this._scrollArea) {
             this.actor.add_style_class_name('multi-line-notification');
             this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
@@ -250,34 +382,30 @@ Notification.prototype = {
             this._addBannerBody();
         }
 
-        this._contentArea.add(actor);
+        this._contentArea.add(actor, style ? style : {});
         this._updated();
     },
 
     // addBody:
     // @text: the text
+    // @markup: %true if @text contains pango markup
+    // @style: style to use when adding the actor containing the text
     //
     // Adds a multi-line label containing @text to the notification.
     //
     // Return value: the newly-added label
-    addBody: function(text) {
-        let body = new St.Label();
-        body.clutter_text.line_wrap = true;
-        body.clutter_text.line_wrap_mode = Pango.WrapMode.WORD_CHAR;
-        body.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
+    addBody: function(text, markup, style) {
+        let label = new URLHighlighter(text, true, markup);
 
-        text = text ? _cleanMarkup(text) : '';
-        body.clutter_text.set_markup(text);
-
-        this.addActor(body);
-        return body;
+        this.addActor(label.actor, style);
+        return label.actor;
     },
 
     _addBannerBody: function() {
         if (this._bannerBodyText) {
             let text = this._bannerBodyText;
             this._bannerBodyText = null;
-            this.addBody(text);
+            this.addBody(text, this._bannerBodyMarkup);
         }
     },
 
@@ -349,7 +477,7 @@ Notification.prototype = {
 
         if (this._useActionIcons && Gtk.IconTheme.get_default().has_icon(id)) {
             button.add_style_class_name('notification-icon-button');
-            button.child = St.TextureCache.get_default().load_icon_name(id, St.IconType.SYMBOLIC, BUTTON_ICON_SIZE);
+            button.child = new St.Icon({ icon_name: id });
         } else {
             button.add_style_class_name('notification-button');
             button.label = label;
