@@ -296,15 +296,15 @@ PopupBaseMenuItem.prototype = {
 };
 Signals.addSignalMethods(PopupBaseMenuItem.prototype);
 
-function PopupMenuItem(text) {
-    this._init(text);
+function PopupMenuItem() {
+    this._init.apply(this, arguments);
 }
 
 PopupMenuItem.prototype = {
     __proto__: PopupBaseMenuItem.prototype,
 
-    _init: function (text) {
-        PopupBaseMenuItem.prototype._init.call(this);
+    _init: function (text, params) {
+        PopupBaseMenuItem.prototype._init.call(this, params);
 
         this.label = new St.Label({ text: text });
         this.addActor(this.label);
@@ -566,22 +566,16 @@ PopupImageMenuItem.prototype = {
     _init: function (text, iconName) {
         PopupBaseMenuItem.prototype._init.call(this);
 
-        this._size = 16;
-
         this.label = new St.Label({ text: text });
         this.addActor(this.label);
-        this._imageBin = new St.Bin({ width: this._size, height: this._size });
-        this.addActor(this._imageBin);
+        this._icon = new St.Icon({ style_class: 'popup-menu-icon' });
+        this.addActor(this._icon);
 
         this.setIcon(iconName);
     },
 
     setIcon: function(name) {
-        if (this._imageBin.child)
-            this._imageBin.child.destroy();
-
-        let img = St.TextureCache.get_default().load_icon_name(name, St.IconType.SYMBOLIC, this._size);
-        this._imageBin.set_child(img);
+        this._icon.icon_name = name;
     }
 };
 
@@ -685,8 +679,11 @@ PopupMenu.prototype = {
         }));
     },
 
-    addMenuItem: function(menuItem) {
-        this._box.add(menuItem.actor);
+    addMenuItem: function(menuItem, position) {
+        if (position == undefined)
+            this._box.add(menuItem.actor);
+        else
+            this._box.insert_actor(menuItem.actor, position);
         menuItem._activeChangeId = menuItem.connect('active-changed', Lang.bind(this, function (menuItem, active) {
             if (active && this._activeMenuItem != menuItem) {
                 if (this._activeMenuItem)
