@@ -1,10 +1,9 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const GLib = imports.gi.GLib;
 const Lang = imports.lang;
-const Shell = imports.gi.Shell;
+const Mainloop = imports.mainloop;
 const Signals = imports.signals;
 const St = imports.gi.St;
 const Gettext = imports.gettext.domain('gnome-shell');
@@ -18,6 +17,7 @@ let contactManager;
 let channelDispatcher;
 
 // See Notification.appendMessage
+const SCROLLBACK_IMMEDIATE_TIME = 60; // 1 minute
 const SCROLLBACK_RECENT_TIME = 15 * 60; // 15 minutes
 const SCROLLBACK_RECENT_LENGTH = 20;
 const SCROLLBACK_IDLE_LENGTH = 5;
@@ -213,7 +213,7 @@ ContactManager.prototype = {
             throw new Error('Could not parse connection path ' + connPath);
 
         info.cacheDir = this._cacheDir + '/' + match[1];
-        GLib.mkdir_with_parents(info.cacheDir, 0700);
+        GLib.mkdir_with_parents(info.cacheDir, 0x1c0); // 0x1c0 = octal 0700
 
         // info.names[handle] is @handle's real name
         // info.tokens[handle] is the token for @handle's avatar
@@ -349,7 +349,9 @@ ContactManager.prototype = {
             let uri = GLib.filename_to_uri(file, null);
             iconBox.child = textureCache.load_uri_async(uri, iconBox._size, iconBox._size);
         } else {
-            iconBox.child = textureCache.load_icon_name('stock_person', iconBox._size);
+            iconBox.child = new St.Icon({ icon_name: 'stock_person',
+                                          icon_type: St.IconType.FULLCOLOR,
+                                          icon_size: iconBox._size });
         }
     },
 
@@ -447,7 +449,7 @@ Source.prototype = {
     __proto__:  MessageTray.Source.prototype,
 
     _init: function(accountPath, connPath, channelPath, targetHandle, targetHandleType, targetId) {
-        MessageTray.Source.prototype._init.call(this, targetId, targetId);
+        MessageTray.Source.prototype._init.call(this, targetId);
 
         this._accountPath = accountPath;
 
@@ -477,13 +479,16 @@ Source.prototype = {
         this._receivedId = this._channelText.connect('Received', Lang.bind(this, this._messageReceived));
 
         this._channelText.ListPendingMessagesRemote(false, Lang.bind(this, this._gotPendingMessages));
+
+        this._setSummaryIcon(this.createNotificationIcon());
     },
 
-    createIcon: function(size) {
-        return contactManager.createAvatar(this._conn, this._targetHandle, size);
+    createNotificationIcon: function() {
+        return contactManager.createAvatar(this._conn, this._targetHandle,
+                                           this.ICON_SIZE);
     },
 
-    clicked: function() {
+    _notificationClicked: function(notification) {
         channelDispatcher.EnsureChannelRemote(this._accountPath,
                                               { 'org.freedesktop.Telepathy.Channel.ChannelType': Telepathy.CHANNEL_TEXT_NAME,
                                                 'org.freedesktop.Telepathy.Channel.TargetHandle': this._targetHandle,
@@ -491,8 +496,6 @@ Source.prototype = {
                                               global.get_current_time(),
                                               '',
                                               Lang.bind(this, this._gotChannelRequest));
-
-        MessageTray.Source.prototype.clicked.call(this);
     },
 
     _gotChannelRequest: function (chanReqPath, ex) {
@@ -524,13 +527,13 @@ Source.prototype = {
             Main.messageTray.add(this);
 
         if (!this._notification)
-            this._notification = new Notification(this._targetId, this);
+            this._notification = new Notification(this);
     },
 
     _messageReceived: function(channel, id, timestamp, sender,
                                type, flags, text) {
         this._ensureNotification();
-        this._notification.appendMessage(text);
+        this._notification.appendMessage(text, timestamp);
         this.notify(this._notification);
     },
 
@@ -542,18 +545,18 @@ Source.prototype = {
         let msg, notify;
 
         if (presence == Telepathy.ConnectionPresenceType.AVAILABLE) {
-            msg = _("%s is online.").format(this.name);
+            msg = _("%s is online.").format(this.title);
             notify = (this._presence == Telepathy.ConnectionPresenceType.OFFLINE);
         } else if (presence == Telepathy.ConnectionPresenceType.OFFLINE ||
                    presence == Telepathy.ConnectionPresenceType.EXTENDED_AWAY) {
             presence = Telepathy.ConnectionPresenceType.OFFLINE;
-            msg = _("%s is offline.").format(this.name);
+            msg = _("%s is offline.").format(this.title);
             notify = (this._presence != Telepathy.ConnectionPresenceType.OFFLINE);
         } else if (presence == Telepathy.ConnectionPresenceType.AWAY) {
-            msg = _("%s is away.").format(this.name);
+            msg = _("%s is away.").format(this.title);
             notify = false;
         } else if (presence == Telepathy.ConnectionPresenceType.BUSY) {
-            msg = _("%s is busy.").format(this.name);
+            msg = _("%s is busy.").format(this.title);
             notify = false;
         } else
             return;
@@ -564,46 +567,61 @@ Source.prototype = {
             msg += ' <i>(' + GLib.markup_escape_text(message, -1) + ')</i>';
 
         this._ensureNotification();
-        this._notification.appendMessage(msg, true);
+        this._notification.appendPresence(msg, notify);
         if (notify)
             this.notify(this._notification);
     }
 };
 
-function Notification(id, source) {
-    this._init(id, source);
+function Notification(source) {
+    this._init(source);
 }
 
 Notification.prototype = {
     __proto__:  MessageTray.Notification.prototype,
 
-    _init: function(id, source) {
-        MessageTray.Notification.prototype._init.call(this, id, source, source.title);
-        this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
+    _init: function(source) {
+        MessageTray.Notification.prototype._init.call(this, source, source.title, null, { customContent: true });
 
         this._responseEntry = new St.Entry({ style_class: 'chat-response' });
-        this._responseEntry.clutter_text.connect('key-focus-in', Lang.bind(this, this._onEntryFocused));
         this._responseEntry.clutter_text.connect('activate', Lang.bind(this, this._onEntryActivated));
         this.setActionArea(this._responseEntry);
 
         this._history = [];
+        this._timestampTimeoutId = 0;
     },
 
-    appendMessage: function(text, asTitle) {
-        if (asTitle)
-            this.update(text);
-        else
-            this.update(this.source.title, text);
-        this._append(text, 'chat-received');
+    appendMessage: function(text, timestamp) {
+        this.update(this.source.title, text, { customContent: true });
+        this._append(text, 'chat-received', timestamp);
     },
 
-    _append: function(text, style) {
+    _append: function(text, style, timestamp) {
+        let currentTime = (Date.now() / 1000);
+        if (!timestamp)
+            timestamp = currentTime;
+        let lastMessageTime = -1;
+        if (this._history.length > 0)
+            lastMessageTime = this._history[0].time;
+
+        // Reset the old message timeout
+        if (this._timestampTimeoutId)
+            Mainloop.source_remove(this._timestampTimeoutId);
+
         let body = this.addBody(text);
         body.add_style_class_name(style);
         this.scrollTo(St.Side.BOTTOM);
 
-        let now = new Date().getTime() / 1000;
-        this._history.unshift({ actor: body, time: now });
+        this._history.unshift({ actor: body, time: timestamp, realMessage: true });
+
+        if (timestamp < currentTime - SCROLLBACK_IMMEDIATE_TIME)
+            this._appendTimestamp();
+        else
+            // Schedule a new timestamp in SCROLLBACK_IMMEDIATE_TIME
+            // from the timestamp of the message.
+            this._timestampTimeoutId = Mainloop.timeout_add_seconds(
+                SCROLLBACK_IMMEDIATE_TIME - (currentTime - timestamp),
+                Lang.bind(this, this._appendTimestamp));
 
         if (this._history.length > 1) {
             // Keep the scrollback from growing too long. If the most
@@ -612,71 +630,57 @@ Notification.prototype = {
             // SCROLLBACK_RECENT_LENGTH previous messages. Otherwise
             // we'll keep SCROLLBACK_IDLE_LENGTH messages.
 
-            let lastMessageTime = this._history[1].time;
-            let maxLength = (lastMessageTime < now - SCROLLBACK_RECENT_TIME) ?
+            let maxLength = (lastMessageTime < currentTime - SCROLLBACK_RECENT_TIME) ?
                 SCROLLBACK_IDLE_LENGTH : SCROLLBACK_RECENT_LENGTH;
-            if (this._history.length > maxLength) {
-                let expired = this._history.splice(maxLength);
+            let filteredHistory = this._history.filter(function(item) { return item.realMessage });
+            if (filteredHistory.length > maxLength) {
+                let lastMessageToKeep = filteredHistory[maxLength];
+                let expired = this._history.splice(this._history.indexOf(lastMessageToKeep));
                 for (let i = 0; i < expired.length; i++)
                     expired[i].actor.destroy();
             }
         }
     },
 
-    _onButtonPress: function(notification, event) {
-        if (!this._active)
-            return false;
+    _appendTimestamp: function() {
+        let lastMessageTime = this._history[0].time;
+        let lastMessageDate = new Date(lastMessageTime * 1000);
 
-        let source = event.get_source ();
-        if (source && notification.contains(source))
-            return false;
+        /* Translators: this is a time format string followed by a date.
+           If applicable, replace %X with a strftime format valid for your
+           locale, without seconds. */
+        let timeLabel = this.addBody(lastMessageDate.toLocaleFormat(_("Sent at %X on %A")), false, { expand: true, x_fill: false, x_align: St.Align.END });
+        timeLabel.add_style_class_name('chat-meta-message');
+        this._history.unshift({ actor: timeLabel, time: lastMessageTime, realMessage: false });
 
-        // @source is outside @notification, which has to mean that
-        // we have a pointer grab, and the user clicked outside the
-        // notification, so we should deactivate.
-        this._deactivate();
-        return true;
+        this._timestampTimeoutId = 0;
+        return false;
     },
 
-    _onEntryFocused: function() {
-        if (this._active)
-            return;
+    appendPresence: function(text, asTitle) {
+        if (asTitle)
+            this.update(text, null, { customContent: true });
+        else
+            this.update(this.source.title, null, { customContent: true });
+        let label = this.addBody(text);
+        label.add_style_class_name('chat-meta-message');
+        this._history.unshift({ actor: label, time: (Date.now() / 1000), realMessage: false});
+    },
 
-        if (!Main.pushModal(this.actor))
-            return;
-        Clutter.grab_pointer(this.actor);
-
-        this._active = true;
-        Main.messageTray.lock();
+    grabFocus: function(lockTray) {
+        // Need to call the base class function first so that
+        // it saves where the key focus was before.
+        MessageTray.Notification.prototype.grabFocus.call(this, lockTray);
+        global.stage.set_key_focus(this._responseEntry.clutter_text);
     },
 
     _onEntryActivated: function() {
         let text = this._responseEntry.get_text();
-        if (text == '') {
-            this._deactivate();
+        if (text == '')
             return;
-        }
 
         this._responseEntry.set_text('');
         this._append(text, 'chat-sent');
         this.source.respond(text);
-    },
-
-    _deactivate: function() {
-        if (this._active) {
-            Clutter.ungrab_pointer(this.actor);
-            Main.popModal(this.actor);
-            global.stage.set_key_focus(null);
-
-            // We have to do this after calling popModal(), because
-            // that will return the keyboard focus to
-            // this._responseEntry (because that's where it was when
-            // pushModal() was called), which will cause
-            // _onEntryFocused() to be called again, but we don't want
-            // it to do anything.
-            this._active = false;
-
-            Main.messageTray.unlock();
-        }
     }
 };
