@@ -1,10 +1,10 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
+const Gio = imports.gi.Gio;
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
-const Shell = imports.gi.Shell;
 const St = imports.gi.St;
 
 const AltTab = imports.ui.altTab;
@@ -13,6 +13,68 @@ const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
 const WINDOW_ANIMATION_TIME = 0.25;
+const DIM_TIME = 0.500;
+const UNDIM_TIME = 0.250;
+
+var dimShader = undefined;
+
+function getDimShader() {
+    if (dimShader === null)
+        return null;
+    if (!dimShader) {
+        let [success, source, length] = GLib.file_get_contents(global.datadir +
+                                                               '/shaders/dim-window.glsl');
+        try {
+            let shader = new Clutter.Shader();
+            shader.set_fragment_source(source, -1);
+            shader.compile();
+
+            dimShader = shader;
+        } catch (e) {
+            log(e.message);
+            dimShader = null;
+        }
+    }
+    return dimShader;
+}
+
+function WindowDimmer(actor) {
+    this._init(actor);
+}
+
+WindowDimmer.prototype = {
+    _init: function(actor) {
+        this.actor = actor;
+    },
+
+    set dimFraction(fraction) {
+        this._dimFraction = fraction;
+        let shader = getDimShader();
+        if (!Meta.prefs_get_attach_modal_dialogs() || !shader) {
+            this.actor.set_shader(null);
+            return;
+        }
+        if (fraction > 0.01) {
+            this.actor.set_shader(shader);
+            this.actor.set_shader_param_float('height', this.actor.get_height());
+            this.actor.set_shader_param_float('fraction', fraction);
+        } else
+            this.actor.set_shader(null);
+    },
+
+    get dimFraction() {
+        return this._dimFraction;
+    },
+
+    _dimFraction: 0.0
+};
+
+function getWindowDimmer(texture) {
+    if (!texture._windowDimmer)
+        texture._windowDimmer = new WindowDimmer(texture);
+
+    return texture._windowDimmer;
+}
 
 function WindowManager() {
     this._init();
@@ -29,19 +91,24 @@ WindowManager.prototype = {
         this._mapping = [];
         this._destroying = [];
 
+        this._dimmedWindows = [];
+
         this._switchData = null;
-        this._shellwm.connect('switch-workspace', Lang.bind(this, this._switchWorkspace));
         this._shellwm.connect('kill-switch-workspace', Lang.bind(this, this._switchWorkspaceDone));
+        this._shellwm.connect('kill-window-effects', Lang.bind(this, function (shellwm, actor) {
+            this._minimizeWindowDone(shellwm, actor);
+            this._maximizeWindowDone(shellwm, actor);
+            this._unmaximizeWindowDone(shellwm, actor);
+            this._mapWindowDone(shellwm, actor);
+            this._destroyWindowDone(shellwm, actor);
+        }));
+
+        this._shellwm.connect('switch-workspace', Lang.bind(this, this._switchWorkspace));
         this._shellwm.connect('minimize', Lang.bind(this, this._minimizeWindow));
-        this._shellwm.connect('kill-minimize', Lang.bind(this, this._minimizeWindowDone));
         this._shellwm.connect('maximize', Lang.bind(this, this._maximizeWindow));
-        this._shellwm.connect('kill-maximize', Lang.bind(this, this._maximizeWindowDone));
         this._shellwm.connect('unmaximize', Lang.bind(this, this._unmaximizeWindow));
-        this._shellwm.connect('kill-unmaximize', Lang.bind(this, this._unmaximizeWindowDone));
         this._shellwm.connect('map', Lang.bind(this, this._mapWindow));
-        this._shellwm.connect('kill-map', Lang.bind(this, this._mapWindowDone));
         this._shellwm.connect('destroy', Lang.bind(this, this._destroyWindow));
-        this._shellwm.connect('kill-destroy', Lang.bind(this, this._destroyWindowDone));
 
         this._workspaceSwitcherPopup = null;
         this.setKeybindingHandler('switch_to_workspace_left', Lang.bind(this, this._showWorkspaceSwitcher));
@@ -49,6 +116,15 @@ WindowManager.prototype = {
         this.setKeybindingHandler('switch_to_workspace_up', Lang.bind(this, this._showWorkspaceSwitcher));
         this.setKeybindingHandler('switch_to_workspace_down', Lang.bind(this, this._showWorkspaceSwitcher));
         this.setKeybindingHandler('switch_windows', Lang.bind(this, this._startAppSwitcher));
+
+        Main.overview.connect('showing', Lang.bind(this, function() {
+            for (let i = 0; i < this._dimmedWindows.length; i++)
+                this._undimParentWindow(this._dimmedWindows[i], true);
+        }));
+        Main.overview.connect('hiding', Lang.bind(this, function() {
+            for (let i = 0; i < this._dimmedWindows.length; i++)
+                this._dimParentWindow(this._dimmedWindows[i], true);
+        }));
     },
 
     setKeybindingHandler: function(keybinding, handler){
@@ -64,7 +140,7 @@ WindowManager.prototype = {
     _shouldAnimate : function(actor) {
         if (Main.overview.visible)
             return false;
-        if (actor && (actor.get_window_type() != Meta.CompWindowType.NORMAL))
+        if (actor && (actor.meta_window.get_window_type() != Meta.WindowType.NORMAL))
             return false;
         return true;
     },
@@ -146,7 +222,108 @@ WindowManager.prototype = {
     _unmaximizeWindowDone : function(shellwm, actor) {
     },
 
+    _parentHasOtherAttachedDialog: function(parent, self) {
+        var count = 0;
+        parent.foreach_transient(function(win) {
+            if (win.get_window_type() == Meta.WindowType.MODAL_DIALOG && win != self)
+                count++;
+            return false;
+        });
+        return count != 0;
+    },
+
+    _markParentWindowAsDimmable: function(actor, animate) {
+        if (Meta.prefs_get_attach_modal_dialogs()) {
+            this._dimmedWindows.push(actor);
+            if (this._shouldAnimate())
+                this._dimParentWindow(actor, animate);
+        }
+    },
+
+    _unmarkParentWindowAsDimmable: function(actor, animate) {
+        if (!Main.overview.visible)
+            this._undimParentWindow(actor, true);
+        this._dimmedWindows = this._dimmedWindows.filter(function(win) {
+            return win != actor;
+        });
+    },
+
+    _dimParentWindow: function(actor, animate) {
+        let meta = actor.get_meta_window();
+        let parent = meta.get_transient_for();
+        if (!parent)
+            return;
+        let parentActor = parent.get_compositor_private();
+        if (!parentActor || this._parentHasOtherAttachedDialog(parent, meta))
+            return;
+        let texture = parentActor.get_texture();
+        if (animate)
+            Tweener.addTween(getWindowDimmer(texture),
+                             { dimFraction: 1.0,
+                               time: DIM_TIME,
+                               transition: 'linear'
+                             });
+        else
+            getWindowDimmer(texture).dimFraction = 1.0;
+    },
+
+    _undimParentWindow: function(actor, animate) {
+        let meta = actor.get_meta_window();
+        let parent = meta.get_transient_for();
+        if (!parent)
+            return;
+        let parentActor = parent.get_compositor_private();
+        if (!parentActor || this._parentHasOtherAttachedDialog(parent, meta))
+            return;
+        let texture = parentActor.get_texture();
+        if (animate)
+            Tweener.addTween(getWindowDimmer(texture),
+                             { dimFraction: 0.0,
+                               time: UNDIM_TIME,
+                               transition: 'linear'
+                             });
+        else
+            getWindowDimmer(texture).dimFraction = 0.0;
+    },
+
     _mapWindow : function(shellwm, actor) {
+        actor._windowType = actor.meta_window.get_window_type();
+        actor._notifyWindowTypeSignalId = actor.meta_window.connect('notify::window-type', Lang.bind(this, function () {
+            let type = actor.meta_window.get_window_type();
+            if (type == actor._windowType)
+                return;
+            if (type == Meta.WindowType.MODAL_DIALOG)
+                this._markParentWindowAsDimmable(actor, true);
+            else if (actor._windowType == Meta.WindowType.MODAL_DIALOG)
+                this._unmarkParentWindowAsDimmable(actor, true);
+
+            actor._windowType = type;
+        }));
+        if (actor.meta_window.get_window_type() == Meta.WindowType.MODAL_DIALOG
+            && Meta.prefs_get_attach_modal_dialogs()
+            && actor.get_meta_window().get_transient_for()) {
+            this._markParentWindowAsDimmable(actor, true);
+            if (this._shouldAnimate()) {
+                actor.set_scale(1.0, 0.0);
+                actor.show();
+                this._mapping.push(actor);
+
+                Tweener.addTween(actor,
+                                 { scale_y: 1,
+                                   time: WINDOW_ANIMATION_TIME,
+                                   transition: "easeOutQuad",
+                                   onComplete: this._mapWindowDone,
+                                   onCompleteScope: this,
+                                   onCompleteParams: [shellwm, actor],
+                                   onOverwrite: this._mapWindowOverwrite,
+                                   onOverwriteScope: this,
+                                   onOverwriteParams: [shellwm, actor]
+                                 });
+                return;
+            }
+            shellwm.completed_map(actor);
+            return;
+        }
         if (!this._shouldAnimate(actor)) {
             shellwm.completed_map(actor);
             return;
@@ -185,10 +362,51 @@ WindowManager.prototype = {
     },
 
     _destroyWindow : function(shellwm, actor) {
+        let parent = actor.meta_window.get_transient_for();
+        if (actor._notifyWindowTypeSignalId) {
+            actor.meta_window.disconnect(actor._notifyWindowTypeSignalId);
+            actor._notifyWindowTypeSignalId = 0;
+        }
+        while (actor.meta_window.get_window_type() == Meta.WindowType.MODAL_DIALOG
+               && parent) {
+            this._unmarkParentWindowAsDimmable(actor, true);
+            if (!Meta.prefs_get_attach_modal_dialogs()
+                || !this._shouldAnimate())
+                break;
+            actor.set_scale(1.0, 1.0);
+            actor.show();
+            this._destroying.push(actor);
+
+            actor._parentDestroyId = parent.connect('unmanaged', Lang.bind(this, function () {
+                Tweener.removeTweens(actor);
+                this._destroyWindowDone(shellwm, actor);
+            }));
+
+            Tweener.addTween(actor,
+                             { scale_y: 0,
+                               time: WINDOW_ANIMATION_TIME,
+                               transition: "easeOutQuad",
+                               onComplete: this._destroyWindowDone,
+                               onCompleteScope: this,
+                               onCompleteParams: [shellwm, actor],
+                               onOverwrite: this._destroyWindowDone,
+                               onOverwriteScope: this,
+                               onOverwriteParams: [shellwm, actor]
+                             });
+            return;
+        }
         shellwm.completed_destroy(actor);
     },
-    
+
     _destroyWindowDone : function(shellwm, actor) {
+        if (this._removeEffect(this._destroying, actor)) {
+            let parent = actor.get_meta_window().get_transient_for();
+            if (parent && actor._parentDestroyId) {
+                parent.disconnect(actor._parentDestroyId);
+                actor._parentDestroyId = 0;
+            }
+            shellwm.completed_destroy(actor);
+        }
     },
 
     _switchWorkspace : function(shellwm, from, to, direction) {
@@ -197,7 +415,7 @@ WindowManager.prototype = {
             return;
         }
 
-        let windows = shellwm.get_switch_workspace_actors();
+        let windows = global.get_window_actors();
 
         /* @direction is the direction that the "camera" moves, so the
          * screen contents have to move one screen's worth in the
