@@ -13,7 +13,7 @@
 #include "shell-tray-manager.h"
 #include "na-tray-manager.h"
 
-#include "shell-gtk-embed.h"
+#include "shell-tray-icon.h"
 #include "shell-embedded-window.h"
 #include "shell-global.h"
 
@@ -30,7 +30,6 @@ typedef struct {
   GtkWidget *socket;
   GtkWidget *window;
   ClutterActor *actor;
-  gboolean emitted_plugged;
 } ShellTrayManagerChild;
 
 enum {
@@ -61,11 +60,13 @@ free_tray_icon (gpointer data)
 {
   ShellTrayManagerChild *child = data;
 
-  gtk_widget_hide (child->window);
   gtk_widget_destroy (child->window);
-  g_signal_handlers_disconnect_matched (child->actor, G_SIGNAL_MATCH_DATA,
-                                        0, 0, NULL, NULL, child);
-  g_object_unref (child->actor);
+  if (child->actor)
+    {
+      g_signal_handlers_disconnect_matched (child->actor, G_SIGNAL_MATCH_DATA,
+                                            0, 0, NULL, NULL, child);
+      g_object_unref (child->actor);
+    }
   g_slice_free (ShellTrayManagerChild, child);
 }
 
@@ -159,10 +160,9 @@ shell_tray_manager_class_init (ShellTrayManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ShellTrayManagerClass, tray_icon_added),
                   NULL, NULL,
-                  gi_cclosure_marshal_generic,
-                  G_TYPE_NONE, 2,
-                  CLUTTER_TYPE_ACTOR,
-                  G_TYPE_STRING);
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1,
+                  CLUTTER_TYPE_ACTOR);
   shell_tray_manager_signals[TRAY_ICON_REMOVED] =
     g_signal_new ("tray-icon-removed",
                   G_TYPE_FROM_CLASS (klass),
@@ -220,116 +220,54 @@ shell_tray_manager_manage_stage (ShellTrayManager *manager,
   else
     stage_window = gdk_window_foreign_new (stage_xwindow);
 
-  screen = gdk_drawable_get_screen (stage_window);
+  screen = gdk_window_get_screen (stage_window);
 
   g_object_unref (stage_window);
 
   na_tray_manager_manage_screen (manager->priv->na_manager, screen);
 }
 
-static GdkPixmap *
-create_bg_pixmap (GdkColormap  *colormap,
-                  ClutterColor *color)
-{
-  GdkScreen *screen = gdk_colormap_get_screen (colormap);
-  GdkVisual *visual = gdk_colormap_get_visual (colormap);
-  GdkPixmap *pixmap = gdk_pixmap_new (gdk_screen_get_root_window (screen),
-                                      1, 1,
-                                      visual->depth);
-  cairo_t *cr;
-
-  gdk_drawable_set_colormap (pixmap, colormap);
-
-  cr = gdk_cairo_create (pixmap);
-  cairo_set_source_rgb (cr,
-                        color->red / 255.,
-                        color->green / 255.,
-                        color->blue / 255.);
-  cairo_paint (cr);
-  cairo_destroy (cr);
-
-  return pixmap;
-}
-
 static void
 shell_tray_manager_child_on_realize (GtkWidget             *widget,
                                      ShellTrayManagerChild *child)
 {
-  GdkPixmap *bg_pixmap;
-
   /* If the tray child is using an RGBA colormap (and so we have real
    * transparency), we don't need to worry about the background. If
-   * not, we obey the bg-color property by creating a 1x1 pixmap of
+   * not, we obey the bg-color property by creating a cairo pattern of
    * that color and setting it as our background. Then "parent-relative"
    * background on the socket and the plug within that will cause
    * the icons contents to appear on top of our background color.
    */
   if (!na_tray_child_has_alpha (NA_TRAY_CHILD (child->socket)))
     {
-      bg_pixmap = create_bg_pixmap (gtk_widget_get_colormap (widget),
-                                    &child->manager->priv->bg_color);
-      gdk_window_set_back_pixmap (gtk_widget_get_window (widget),
-                                  bg_pixmap, FALSE);
-      g_object_unref (bg_pixmap);
+      ClutterColor color = child->manager->priv->bg_color;
+      cairo_pattern_t *bg_pattern;
+
+      bg_pattern = cairo_pattern_create_rgb (color.red / 255.,
+                                             color.green / 255.,
+                                             color.blue / 255.);
+      gdk_window_set_background_pattern (gtk_widget_get_window (widget),
+                                         bg_pattern);
+
+      cairo_pattern_destroy (bg_pattern);
     }
 }
 
-static char *
-get_lowercase_wm_class_from_socket (ShellTrayManager  *manager,
-                                    GtkSocket         *socket)
-{
-  GdkWindow *window;
-  MetaScreen *screen;
-  MetaDisplay *display;
-  XClassHint class_hint;
-  gboolean success;
-  char *result;
-
-  window = gtk_socket_get_plug_window (socket);
-  g_return_val_if_fail (window != NULL, NULL);
-
-  screen = shell_global_get_screen (shell_global_get ());
-  display = meta_screen_get_display (screen);
-
-  gdk_error_trap_push ();
-
-  success = XGetClassHint (meta_display_get_xdisplay (display), GDK_WINDOW_XWINDOW (window), &class_hint);
-
-  gdk_error_trap_pop ();
-
-  if (!success)
-    return NULL;
-
-  result = g_ascii_strdown (class_hint.res_class, -1);
-  XFree (class_hint.res_name);
-  XFree (class_hint.res_class);
-  return result;
-}
-
 static void
 on_plug_added (GtkSocket        *socket,
                ShellTrayManager *manager)
 {
   ShellTrayManagerChild *child;
-  char *wm_class;
+
+  g_signal_handlers_disconnect_by_func (socket, on_plug_added, manager);
 
   child = g_hash_table_lookup (manager->priv->icons, socket);
-  /* Only emit this signal once; the point of waiting until we
-   * get the first plugged notification is to be able to get the WM_CLASS
-   * from the child window.  But we don't want to emit this signal twice
-   * if for some reason the socket gets replugged.
-   */
-  if (child->emitted_plugged)
-    return;
-  child->emitted_plugged = TRUE;
 
-  wm_class = get_lowercase_wm_class_from_socket (manager, socket);
-  if (!wm_class)
-    return;
+  child->actor = shell_tray_icon_new (SHELL_EMBEDDED_WINDOW (child->window));
+  g_object_ref_sink (child->actor);
 
   g_signal_emit (manager, shell_tray_manager_signals[TRAY_ICON_ADDED], 0,
-                 child->actor, wm_class);
-  g_free (wm_class);
+                 child->actor);
 }
 
 static void
@@ -338,7 +276,6 @@ na_tray_icon_added (NaTrayManager *na_manager, GtkWidget *socket,
 {
   ShellTrayManager *manager = user_data;
   GtkWidget *win;
-  ClutterActor *icon;
   ShellTrayManagerChild *child;
 
   /* We don't need the NaTrayIcon to be composited on the window we
@@ -349,12 +286,12 @@ na_tray_icon_added (NaTrayManager *na_manager, GtkWidget *socket,
    */
   na_tray_child_set_composited (NA_TRAY_CHILD (socket), FALSE);
 
-  win = shell_embedded_window_new ();
+  win = shell_embedded_window_new (manager->priv->stage);
   gtk_container_add (GTK_CONTAINER (win), socket);
 
-  /* The colormap of the socket matches that of its contents; make
+  /* The visual of the socket matches that of its contents; make
    * the window we put it in match that as well */
-  gtk_widget_set_colormap (win, gtk_widget_get_colormap (socket));
+  gtk_widget_set_visual (win, gtk_widget_get_visual (socket));
 
   child = g_slice_new0 (ShellTrayManagerChild);
   child->manager = manager;
@@ -366,9 +303,6 @@ na_tray_icon_added (NaTrayManager *na_manager, GtkWidget *socket,
 
   gtk_widget_show_all (win);
 
-  icon = shell_gtk_embed_new (SHELL_EMBEDDED_WINDOW (win));
-
-  child->actor = g_object_ref (icon);
   g_hash_table_insert (manager->priv->icons, socket, child);
 
   g_signal_connect (socket, "plug-added", G_CALLBACK (on_plug_added), manager);
@@ -384,8 +318,14 @@ na_tray_icon_removed (NaTrayManager *na_manager, GtkWidget *socket,
   child = g_hash_table_lookup (manager->priv->icons, socket);
   g_return_if_fail (child != NULL);
 
-  g_signal_emit (manager,
-                 shell_tray_manager_signals[TRAY_ICON_REMOVED], 0,
-                 child->actor);
+  if (child->actor != NULL)
+    {
+      /* Only emit signal if a corresponding tray-icon-added signal was emitted,
+         that is, if embedding did not fail and we got a plug-added
+      */
+      g_signal_emit (manager,
+                     shell_tray_manager_signals[TRAY_ICON_REMOVED], 0,
+                     child->actor);
+    }
   g_hash_table_remove (manager->priv->icons, socket);
 }
