@@ -30,6 +30,9 @@ GType st_corner_get_type (void) G_GNUC_CONST;
 GType st_text_decoration_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_TEXT_DECORATION (st_text_decoration_get_type())
 
+GType st_text_align_get_type (void) G_GNUC_CONST;
+#define ST_TYPE_TEXT_ALIGN (st_text_align_get_type())
+
 GType st_gradient_type_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_GRADIENT_TYPE (st_gradient_type_get_type())
 
@@ -37,6 +40,9 @@ GType st_gradient_type_get_type (void) G_GNUC_CONST;
 GType st_align_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_ALIGN (st_align_get_type())
 
+GType st_icon_type_get_type (void) G_GNUC_CONST;
+#define ST_TYPE_ICON_TYPE (st_icon_type_get_type())
+
 /* enumerations from "st/st-widget.h" */
 GType st_text_direction_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_TEXT_DIRECTION (st_text_direction_get_type())
