@@ -12,6 +12,8 @@
 #include <st/st-entry.h>
 #include <st/st-focus-manager.h>
 #include <st/st-group.h>
+#include <st/st-icon.h>
+#include <st/st-icon-colors.h>
 #include <st/st-im-text.h>
 #include <st/st-label.h>
 #include <st/st-overflow-box.h>
@@ -20,7 +22,6 @@
 #include <st/st-scroll-bar.h>
 #include <st/st-scroll-view.h>
 #include <st/st-shadow.h>
-#include <st/st-subtexture.h>
 #include <st/st-table.h>
 #include <st/st-table-child.h>
 #include <st/st-texture-cache.h>
