@@ -4,6 +4,10 @@
  *
  * Copyright 2007 OpenedHand
  * Copyright 2008, 2009 Intel Corporation.
+ * Copyright 2009, 2010 Red Hat, Inc.
+ * Copyright 2009 Abderrahim Kitouni
+ * Copyright 2009, 2010 Florian Müllner
+ * Copyright 2010 Adel Gadllah
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -15,13 +19,7 @@
  * more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
- * Boston, MA 02111-1307, USA.
- *
- * Written by: Emmanuele Bassi <ebassi@openedhand.com>
- *             Thomas Wood <thomas@linux.intel.com>
- *
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -525,6 +523,9 @@ get_root_theme_node (ClutterStage *stage)
  * The theme node is used to access standard and custom CSS
  * properties of the widget.
  *
+ * Note: it is a fatal error to call this on a widget that is
+ *  not been added to a stage.
+ *
  * Return value: (transfer none): the theme node for the widget.
  *   This is owned by the widget. When attributes of the widget
  *   or the environment that affect the styling change (for example
@@ -573,6 +574,29 @@ st_widget_get_theme_node (StWidget *widget)
   return priv->theme_node;
 }
 
+/**
+ * st_widget_peek_theme_node:
+ * @widget: a #StWidget
+ *
+ * Returns the theme node for the widget if it has already been
+ * computed, %NULL if the widget hasn't been added to a  stage or the theme
+ * node hasn't been computed. If %NULL is returned, then ::style-changed
+ * will be reliably emitted before the widget is allocated or painted.
+ *
+ * Return value: (transfer none): the theme node for the widget.
+ *   This is owned by the widget. When attributes of the widget
+ *   or the environment that affect the styling change (for example
+ *   the style_class property of the widget), it will be recreated,
+ *   and the ::style-changed signal will be emitted on the widget.
+ */
+StThemeNode *
+st_widget_peek_theme_node (StWidget *widget)
+{
+  StWidgetPrivate *priv = widget->priv;
+
+  return priv->theme_node;
+}
+
 static gboolean
 st_widget_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
@@ -640,11 +664,36 @@ st_widget_hide (ClutterActor *actor)
 
   /* hide the tooltip, if there is one */
   if (widget->priv->tooltip)
-    st_tooltip_hide (ST_TOOLTIP (widget->priv->tooltip));
+    clutter_actor_hide (CLUTTER_ACTOR (widget->priv->tooltip));
 
   CLUTTER_ACTOR_CLASS (st_widget_parent_class)->hide (actor);
 }
 
+static gboolean
+st_widget_get_paint_volume (ClutterActor *self, ClutterPaintVolume *volume)
+{
+  ClutterActorBox paint_box, alloc_box;
+  StThemeNode *theme_node;
+  ClutterVertex origin;
+
+  /* Setting the paint volume does not make sense when we don't have any allocation */
+  if (!clutter_actor_has_allocation (self))
+    return FALSE;
+
+  theme_node = st_widget_get_theme_node (ST_WIDGET(self));
+  clutter_actor_get_allocation_box (self, &alloc_box);
+  st_theme_node_get_paint_box (theme_node, &alloc_box, &paint_box);
+
+  origin.x = paint_box.x1 - alloc_box.x1;
+  origin.y = paint_box.y1 - alloc_box.y1;
+  origin.z = 0.0f;
+
+  clutter_paint_volume_set_origin (volume, &origin);
+  clutter_paint_volume_set_width (volume, paint_box.x2 - paint_box.x1);
+  clutter_paint_volume_set_height (volume, paint_box.y2 - paint_box.y1);
+
+  return TRUE;
+}
 
 
 static void
@@ -665,6 +714,7 @@ st_widget_class_init (StWidgetClass *klass)
   actor_class->get_preferred_height = st_widget_get_preferred_height;
   actor_class->allocate = st_widget_allocate;
   actor_class->paint = st_widget_paint;
+  actor_class->get_paint_volume = st_widget_get_paint_volume;
   actor_class->parent_set = st_widget_parent_set;
   actor_class->map = st_widget_map;
   actor_class->unmap = st_widget_unmap;
@@ -1511,7 +1561,7 @@ st_widget_show_tooltip (StWidget *widget)
   if (widget->priv->tooltip)
     {
       st_tooltip_set_tip_area (widget->priv->tooltip, &area);
-      st_tooltip_show (widget->priv->tooltip);
+      clutter_actor_show_all (CLUTTER_ACTOR (widget->priv->tooltip));
     }
 }
 
@@ -1528,7 +1578,7 @@ st_widget_hide_tooltip (StWidget *widget)
   g_return_if_fail (ST_IS_WIDGET (widget));
 
   if (widget->priv->tooltip)
-    st_tooltip_hide (widget->priv->tooltip);
+    clutter_actor_hide (CLUTTER_ACTOR (widget->priv->tooltip));
 }
 
 /**
